/***************************************************************************
 *            inspector.c
 *
 *  Fri Jul 23 11:43:41 2004
 *  Copyright  2004  benjie
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <gtk/gtk.h>

#include "support.h"
#include "callbacks.h"
#include "interface.h"
#include "tocdoc.h"
#include "inspector.h"
#include "timeline.h"

extern tlType *tl;


void
on_fileEntry_activate                  (GtkEntry        *field,
                                        gpointer         user_data)
{
    GList       *item;
    tocEntry    *entry;
    
    if(tl->document->selection) {
        g_mutex_lock(tl->cursor.mutex);
        item = g_list_first(tl->document->selection);
        while (item) {
            entry = (tocEntry*)((GList *)item->data)->data;
            if (entry->fileType != teSilence)
                tocEntrySetFilename(entry, (gchar *)gtk_entry_get_text(field));
            item = g_list_next(item);
        }
        tl->cursor.changedFlag = 1;
        g_mutex_unlock(tl->cursor.mutex);
    }
    redrawTimeline(tl);
    updateInspector();
}


void
on_lengthEntry_activate                (GtkEntry     *field,
                                        gpointer      user_data)
{
    gint        value;
    GList       *item;
    tocEntry    *entry;
    
    value = cdTimeToGint((gchar *)gtk_entry_get_text(field));
    if (tl->document->selection && value != -1) {
        item = g_list_first(tl->document->selection);
        g_mutex_lock(tl->cursor.mutex);
            while (item) {
                entry = (tocEntry*)((GList *)item->data)->data;
                tocDocResizeEntry(tl->document, entry, value);
                item = g_list_next(item);
            }
            tl->cursor.changedFlag = 1;
            tocDocSetDirty(tl->document, TRUE);
        g_mutex_unlock(tl->cursor.mutex);
        redrawTimeline(tl);
        tlSetScroller(tl, -1);
        updateInspector();
    } else {
        updateInspector();
    }
}


void
on_pregapEntry_activate                (GtkEntry     *field,
                                        gpointer      user_data)
{
    gint        value;
    GList       *item;
    tocEntry    *entry;
    
    value = cdTimeToGint((gchar *)gtk_entry_get_text(field));
    if (tl->document->selection && value != -1) {
        item = g_list_first(tl->document->selection);
        g_mutex_lock(tl->cursor.mutex);
            while (item) {
                entry = (tocEntry*)((GList *)item->data)->data;
                if (entry->fileType != teSilence)
                    tocDocResizePregap(tl->document, entry, value);
                item = g_list_next(item);
            }
            tl->cursor.changedFlag = 1;
            tocDocSetDirty(tl->document, TRUE);
        g_mutex_unlock(tl->cursor.mutex);
        redrawTimeline(tl);
        tlSetScroller(tl, -1);
        updateInspector();
    } else {
        updateInspector();
    }
}


void
on_offsetEntry_activate                (GtkEntry     *field,
                                        gpointer      user_data)
{
    gint        value;
    GList       *item;
    tocEntry    *entry;
    
    value = cdTimeToGint((gchar *)gtk_entry_get_text(field));
    if (tl->document->selection && value != -1) {
        item = g_list_first(tl->document->selection);
        g_mutex_lock(tl->cursor.mutex);
            while (item) {
                entry = (tocEntry*)((GList *)item->data)->data;
                if (entry->fileType != teSilence)
                    entry->offset_F = value;
                item = g_list_next(item);
            }
            tl->cursor.changedFlag = 1;
            tocDocSetDirty(tl->document, TRUE);
        g_mutex_unlock(tl->cursor.mutex);
        redrawTimeline(tl);
        updateInspector();
    } else {
        updateInspector();
    }
}


void updateInspector(void)
{
    GtkWidget   *field;
    gchar       timeBuf[64];
    tocEntry    *entry;
    
    if(g_list_length(tl->document->selection) == 0){
        field = lookup_widget(tl->timeline, "fileEntry");
        gtk_entry_set_text((GtkEntry *)field, "");
        gtk_widget_set_sensitive(field, FALSE);

        field = lookup_widget(tl->timeline, "startEntry");
        gtk_entry_set_text((GtkEntry *)field, "");

        field = lookup_widget(tl->timeline, "endEntry");
        gtk_entry_set_text((GtkEntry *)field, "");

        field = lookup_widget(tl->timeline, "lengthEntry");
        gtk_entry_set_text((GtkEntry *)field, "");
        gtk_widget_set_sensitive(field, FALSE);

        field = lookup_widget(tl->timeline, "pregapEntry");
        gtk_entry_set_text((GtkEntry *)field, "");
        gtk_widget_set_sensitive(field, FALSE);

        field = lookup_widget(tl->timeline, "offsetEntry");
        gtk_entry_set_text((GtkEntry *)field, "");
        gtk_widget_set_sensitive(field, FALSE);

        gtk_widget_set_sensitive(lookup_widget(tl->timeline, "browse"), FALSE);
    }
    else if(g_list_length(tl->document->selection) == 1) {
        entry = ((GList *)g_list_first(tl->document->selection)->data)->data;
        
        field = lookup_widget(tl->timeline, "fileEntry");
        if (entry->fileType != teSilence) {
            gtk_entry_set_text((GtkEntry *)field, (entry->filename) ? entry->filename : "");
            gtk_widget_set_sensitive(field, TRUE);
        } else {
            gtk_entry_set_text((GtkEntry *)field, "silence");
            gtk_widget_set_sensitive(field, FALSE);
        }
        field = lookup_widget(tl->timeline, "startEntry");
        gintToCdTime(timeBuf, entry->start_F);
        gtk_entry_set_text((GtkEntry *)field, timeBuf);

        field = lookup_widget(tl->timeline, "endEntry");
        gintToCdTime(timeBuf, entry->start_F + entry->audioLength_F);
        gtk_entry_set_text((GtkEntry *)field, timeBuf);

        field = lookup_widget(tl->timeline, "lengthEntry");
        gintToCdTime(timeBuf, entry->audioLength_F);
        gtk_entry_set_text((GtkEntry *)field, timeBuf);
        gtk_widget_set_sensitive(field, TRUE);

        field = lookup_widget(tl->timeline, "pregapEntry");
        gintToCdTime(timeBuf, entry->pregapLength_F);
        gtk_entry_set_text((GtkEntry *)field, timeBuf);
        gtk_widget_set_sensitive(field, (entry->fileType != teSilence));

        field = lookup_widget(tl->timeline, "offsetEntry");
        gintToCdTime(timeBuf, entry->offset_F);
        gtk_entry_set_text((GtkEntry *)field, timeBuf);
        gtk_widget_set_sensitive(field, (entry->fileType != teSilence));

        gtk_widget_set_sensitive(lookup_widget(tl->timeline, "browse"), TRUE);
    }
    else {
        field = lookup_widget(tl->timeline, "fileEntry");
        gtk_entry_set_text((GtkEntry *)field, "");
        gtk_widget_set_sensitive(field, TRUE);

        field = lookup_widget(tl->timeline, "startEntry");
        gtk_entry_set_text((GtkEntry *)field, "");

        field = lookup_widget(tl->timeline, "endEntry");
        gtk_entry_set_text((GtkEntry *)field, "");

        field = lookup_widget(tl->timeline, "lengthEntry");
        gtk_entry_set_text((GtkEntry *)field, "");
        gtk_widget_set_sensitive(field, TRUE);

        field = lookup_widget(tl->timeline, "pregapEntry");
        gtk_entry_set_text((GtkEntry *)field, "");
        gtk_widget_set_sensitive(field, TRUE);

        field = lookup_widget(tl->timeline, "offsetEntry");
        gtk_entry_set_text((GtkEntry *)field, "");
        gtk_widget_set_sensitive(field, TRUE);

        gtk_widget_set_sensitive(lookup_widget(tl->timeline, "browse"), TRUE);
    }
}


void gintToCdTime(gchar *buf, glong t)
{
    glong min, sec, frame;

    frame = t % 75;
    min = t/(60*75);
    sec = t/75 - min*60;
    sprintf(buf, "%d:%02d:%02d", (int)min, (int)sec, (int)frame); 
}


// Returns -1 for invalid string
glong cdTimeToGint(gchar *buf)
{
    glong min, sec, frame;
    gint result;
    
    result = sscanf(buf, "%lu%*1s%lu%*1s%lu", &min, &sec, &frame);
    if (result == 3)
        return ((min * 60 * 75) + (sec * 75) + frame);

    return -1;    
}
